import sys
from typing import Any, Dict, Generic, Optional, Type, TypeVar, Union

from django.db import models
from django.forms.forms import BaseForm
from django.forms.models import BaseModelForm
from django.http import HttpRequest, HttpResponse
from django.utils.datastructures import _ListOrTuple
from django.views.generic.base import ContextMixin, TemplateResponseMixin, View
from django.views.generic.detail import BaseDetailView, SingleObjectMixin, SingleObjectTemplateResponseMixin

if sys.version_info < (3, 8):
    from typing_extensions import Literal
else:
    from typing import Literal

_FormT = TypeVar("_FormT", bound=BaseForm)
_ModelFormT = TypeVar("_ModelFormT", bound=BaseModelForm)
_M = TypeVar("_M", bound=models.Model)

class FormMixin(Generic[_FormT], ContextMixin):
    initial: Dict[str, Any] = ...
    form_class: Optional[Type[_FormT]] = ...
    success_url: Optional[str] = ...
    prefix: Optional[str] = ...
    def get_initial(self) -> Dict[str, Any]: ...
    def get_prefix(self) -> Optional[str]: ...
    def get_form_class(self) -> Type[_FormT]: ...
    def get_form(self, form_class: Optional[Type[_FormT]] = ...) -> _FormT: ...
    def get_form_kwargs(self) -> Dict[str, Any]: ...
    def get_success_url(self) -> str: ...
    def form_valid(self, form: _FormT) -> HttpResponse: ...
    def form_invalid(self, form: _FormT) -> HttpResponse: ...
    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]: ...

class ModelFormMixin(Generic[_M, _ModelFormT], FormMixin[_ModelFormT], SingleObjectMixin[_M]):
    fields: Optional[Union[_ListOrTuple[str], Literal["__all__"]]] = ...
    def get_form_class(self) -> Type[_ModelFormT]: ...
    def get_form_kwargs(self) -> Dict[str, Any]: ...
    def get_success_url(self) -> str: ...
    def form_valid(self, form: _ModelFormT) -> HttpResponse: ...

class ProcessFormView(View):
    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def put(self, *args: Any, **kwargs: Any) -> HttpResponse: ...

class BaseFormView(FormMixin[_FormT], ProcessFormView): ...
class FormView(TemplateResponseMixin, BaseFormView[_FormT]): ...

class BaseCreateView(ModelFormMixin[_M, _ModelFormT], ProcessFormView):
    object: None
    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...

class CreateView(SingleObjectTemplateResponseMixin, BaseCreateView[_M, _ModelFormT]):
    template_name_suffix: str = ...

class BaseUpdateView(ModelFormMixin[_M, _ModelFormT], ProcessFormView):
    object: _M
    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...

class UpdateView(SingleObjectTemplateResponseMixin, BaseUpdateView[_M, _ModelFormT]):
    template_name_suffix: str = ...

class DeletionMixin(Generic[_M]):
    success_url: Optional[str] = ...
    object: _M
    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def delete(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def get_success_url(self) -> str: ...

class BaseDeleteView(DeletionMixin[_M], BaseDetailView[_M]):
    object: _M

class DeleteView(SingleObjectTemplateResponseMixin, BaseDeleteView[_M]):
    object: _M
    template_name_suffix: str = ...



import configparser
import textwrap
from functools import partial
from pathlib import Path
from typing import Any, Callable, Dict, NoReturn, Optional

import tomli

INI_USAGE = """
(config)
...
[mypy.plugins.django_stubs]
    django_settings_module: str (required)
...
"""
TOML_USAGE = """
(config)
...
[tool.django-stubs]
django_settings_module = str (required)
...
"""
INVALID_FILE = "mypy config file is not specified or found"
COULD_NOT_LOAD_FILE = "could not load configuration file"
MISSING_SECTION = "no section [{section}] found".format
MISSING_DJANGO_SETTINGS = "missing required 'django_settings_module' config"
INVALID_SETTING = "invalid {key!r}: the setting must be a boolean".format


def exit_with_error(msg: str, is_toml: bool = False) -> NoReturn:
    """Using mypy's argument parser, raise `SystemExit` to fail hard if validation fails.

    Considering that the plugin's startup duration is around double as long as mypy's, this aims to
    import and construct objects only when that's required - which happens once and terminates the
    run. Considering that most of the runs are successful, there's no need for this to linger in the
    global scope.
    """
    from mypy.main import CapturableArgumentParser

    handler = CapturableArgumentParser(
        prog="(django-stubs) mypy", usage=textwrap.dedent(TOML_USAGE if is_toml else INI_USAGE)
    )
    handler.error(msg)


class DjangoPluginConfig:
    __slots__ = ("django_settings_module",)
    django_settings_module: str

    def __init__(self, config_file: Optional[str]) -> None:
        if not config_file:
            exit_with_error(INVALID_FILE)

        filepath = Path(config_file)
        if not filepath.is_file():
            exit_with_error(INVALID_FILE)

        if filepath.suffix.lower() == ".toml":
            self.parse_toml_file(filepath)
        else:
            self.parse_ini_file(filepath)

    def parse_toml_file(self, filepath: Path) -> None:
        toml_exit: Callable[[str], NoReturn] = partial(exit_with_error, is_toml=True)
        try:
            with filepath.open(mode="rb") as f:
                data = tomli.load(f)
        except (tomli.TOMLDecodeError, OSError):
            toml_exit(COULD_NOT_LOAD_FILE)

        try:
            config: Dict[str, Any] = data["tool"]["django-stubs"]
        except KeyError:
            toml_exit(MISSING_SECTION(section="tool.django-stubs"))

        if "django_settings_module" not in config:
            toml_exit(MISSING_DJANGO_SETTINGS)

        self.django_settings_module = config["django_settings_module"]
        if not isinstance(self.django_settings_module, str):
            toml_exit("invalid 'django_settings_module': the setting must be a string")

    def parse_ini_file(self, filepath: Path) -> None:
        parser = configparser.ConfigParser()
        try:
            with filepath.open(encoding="utf-8") as f:
                parser.read_file(f, source=str(filepath))
        except OSError:
            exit_with_error(COULD_NOT_LOAD_FILE)

        section = "mypy.plugins.django-stubs"
        if not parser.has_section(section):
            exit_with_error(MISSING_SECTION(section=section))

        if not parser.has_option(section, "django_settings_module"):
            exit_with_error(MISSING_DJANGO_SETTINGS)

        self.django_settings_module = parser.get(section, "django_settings_module").strip("'\"")



from argparse import ArgumentParser
from typing import Any, Optional, Sequence, Tuple

from django.core.management.base import BaseCommand

class TemplateCommand(BaseCommand):
    url_schemes: Sequence[str] = ...
    rewrite_template_suffixes: Sequence[Tuple[str, str]] = ...
    app_or_project: str
    a_or_an: str
    paths_to_remove: Sequence[Any]
    verbosity: Any = ...
    def add_arguments(self, parser: ArgumentParser) -> None: ...
    def handle(self, app_or_project: str, name: str, target: Optional[str] = ..., **options: Any) -> None: ...  # type: ignore
    def handle_template(self, template: Optional[str], subdir: Optional[str]) -> str: ...
    def validate_name(self, name: str, name_or_dir: str = ...) -> None: ...
    def download(self, url: str) -> str: ...
    def splitext(self, the_path: str) -> Tuple[str, str]: ...
    def extract(self, filename: str) -> str: ...
    def is_url(self, template: str) -> bool: ...
    def make_writeable(self, filename: str) -> None: ...



# Stubs for django.conf.urls (Python 3.5)
from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Union, overload

from django.http.response import HttpResponse, HttpResponseBase
from django.urls import URLPattern, URLResolver
from django.urls import include as include

handler400: Union[str, Callable[..., HttpResponse]] = ...
handler403: Union[str, Callable[..., HttpResponse]] = ...
handler404: Union[str, Callable[..., HttpResponse]] = ...
handler500: Union[str, Callable[..., HttpResponse]] = ...

IncludedURLConf = Tuple[Sequence[Union[URLResolver, URLPattern]], Optional[str], Optional[str]]

# Deprecated
@overload
def url(
    regex: str, view: Callable[..., HttpResponseBase], kwargs: Optional[Dict[str, Any]] = ..., name: Optional[str] = ...
) -> URLPattern: ...
@overload
def url(
    regex: str, view: IncludedURLConf, kwargs: Optional[Dict[str, Any]] = ..., name: Optional[str] = ...
) -> URLResolver: ...
@overload
def url(
    regex: str,
    view: Sequence[Union[URLResolver, str]],
    kwargs: Optional[Dict[str, Any]] = ...,
    name: Optional[str] = ...,
) -> URLResolver: ...



import os
from pathlib import Path
from typing import List, Tuple, Union

from django.core.management.base import BaseCommand as BaseCommand
from django.core.management.base import CommandError as CommandError
from django.core.management.base import CommandParser as CommandParser
from django.core.management.utils import find_command as find_command
from django.core.management.utils import popen_wrapper as popen_wrapper
from django.utils._os import _PathCompatible

def has_bom(fn: Path) -> bool: ...
def is_writable(path: _PathCompatible) -> bool: ...

class Command(BaseCommand):
    program: str = ...
    program_options: List[str] = ...
    verbosity: int = ...
    has_errors: bool = ...
    def compile_messages(self, locations: List[Tuple[_PathCompatible, _PathCompatible]]) -> None: ...



import datetime
import sys
from typing import Any, Dict, Iterable, Iterator, List, Mapping, Optional, Sequence, Tuple, Type, Union

from django.core.files.base import File
from django.db.models.fields import _FieldChoices
from django.forms.renderers import BaseRenderer
from django.forms.utils import _DataT, _FilesT
from django.utils.datastructures import _ListOrTuple
from django.utils.functional import _Getter
from django.utils.safestring import SafeString
from typing_extensions import Literal

if sys.version_info < (3, 8):
    from typing_extensions import Protocol
else:
    from typing import Protocol

_OptAttrs = Dict[str, Any]

class MediaOrderConflictWarning(RuntimeWarning): ...

class Media:
    def __init__(
        self,
        media: Optional[type] = ...,
        css: Optional[Dict[str, Sequence[str]]] = ...,
        js: Optional[Sequence[str]] = ...,
    ) -> None: ...
    def render(self) -> SafeString: ...
    def render_js(self) -> List[SafeString]: ...
    def render_css(self) -> Iterable[SafeString]: ...
    def absolute_path(self, path: str) -> str: ...
    def __getitem__(self, name: str) -> Media: ...
    @staticmethod
    def merge(*lists: Iterable[Any]) -> List[Any]: ...
    def __add__(self, other: Media) -> Media: ...

class MediaDefiningClass(type): ...

class Widget(metaclass=MediaDefiningClass):
    needs_multipart_form: bool = ...
    is_localized: bool = ...
    is_required: bool = ...
    supports_microseconds: bool = ...
    attrs: _OptAttrs = ...
    template_name: str
    media: _Getter[Media]
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...
    @property
    def is_hidden(self) -> bool: ...
    def subwidgets(self, name: str, value: Any, attrs: _OptAttrs = ...) -> Iterator[Dict[str, Any]]: ...
    def format_value(self, value: Any) -> Optional[str]: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...
    def render(
        self, name: str, value: Any, attrs: Optional[_OptAttrs] = ..., renderer: Optional[BaseRenderer] = ...
    ) -> SafeString: ...
    def build_attrs(self, base_attrs: _OptAttrs, extra_attrs: Optional[_OptAttrs] = ...) -> Dict[str, Any]: ...
    def value_from_datadict(self, data: _DataT, files: _FilesT, name: str) -> Any: ...
    def value_omitted_from_data(self, data: _DataT, files: _FilesT, name: str) -> bool: ...
    def id_for_label(self, id_: str) -> str: ...
    def use_required_attribute(self, initial: Any) -> bool: ...

class Input(Widget):
    input_type: str = ...
    template_name: str = ...

class TextInput(Input):
    input_type: str = ...
    template_name: str = ...

class NumberInput(Input):
    input_type: str = ...
    template_name: str = ...

class EmailInput(Input):
    input_type: str = ...
    template_name: str = ...

class URLInput(Input):
    input_type: str = ...
    template_name: str = ...

class PasswordInput(Input):
    render_value: bool = ...
    input_type: str = ...
    template_name: str = ...
    def __init__(self, attrs: Optional[_OptAttrs] = ..., render_value: bool = ...) -> None: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...

class HiddenInput(Input):
    choices: _FieldChoices
    input_type: str = ...
    template_name: str = ...

class MultipleHiddenInput(HiddenInput):
    template_name: str = ...

class FileInput(Input):
    input_type: str = ...
    template_name: str = ...
    needs_multipart_form: bool = ...
    def format_value(self, value: Any) -> None: ...
    def value_from_datadict(self, data: _DataT, files: _FilesT, name: str) -> Any: ...
    def value_omitted_from_data(self, data: _DataT, files: _FilesT, name: str) -> bool: ...
    def use_required_attribute(self, initial: Any) -> bool: ...

FILE_INPUT_CONTRADICTION: object

class ClearableFileInput(FileInput):
    clear_checkbox_label: str = ...
    initial_text: str = ...
    input_text: str = ...
    template_name: str = ...
    def clear_checkbox_name(self, name: str) -> str: ...
    def clear_checkbox_id(self, name: str) -> str: ...
    def is_initial(self, value: Optional[Union[File, str]]) -> bool: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...
    def value_from_datadict(self, data: _DataT, files: _FilesT, name: str) -> Any: ...
    def value_omitted_from_data(self, data: _DataT, files: _FilesT, name: str) -> bool: ...

class Textarea(Widget):
    template_name: str = ...
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...

class DateTimeBaseInput(TextInput):
    format_key: str = ...
    format: Optional[str] = ...
    supports_microseconds: bool = ...
    def __init__(self, attrs: Optional[_OptAttrs] = ..., format: Optional[str] = ...): ...

class DateInput(DateTimeBaseInput):
    format_key: str = ...
    template_name: str = ...

class DateTimeInput(DateTimeBaseInput):
    format_key: str = ...
    template_name: str = ...

class TimeInput(DateTimeBaseInput):
    format_key: str = ...
    template_name: str = ...

def boolean_check(v: Any) -> bool: ...

class _CheckCallable(Protocol):
    def __call__(self, __value: Any) -> bool: ...

class CheckboxInput(Input):
    check_test: _CheckCallable = ...
    input_type: str = ...
    template_name: str = ...
    def __init__(self, attrs: Optional[_OptAttrs] = ..., check_test: Optional[_CheckCallable] = ...) -> None: ...

class ChoiceWidget(Widget):
    allow_multiple_selected: bool = ...
    input_type: Optional[str] = ...
    template_name: str = ...
    option_template_name: Optional[str] = ...
    add_id_index: bool = ...
    checked_attribute: Any = ...
    option_inherits_attrs: bool = ...
    choices: _FieldChoices = ...
    def __init__(self, attrs: Optional[_OptAttrs] = ..., choices: _FieldChoices = ...) -> None: ...
    def subwidgets(self, name: str, value: Any, attrs: _OptAttrs = ...) -> Iterator[Dict[str, Any]]: ...
    def options(self, name: str, value: List[str], attrs: Optional[_OptAttrs] = ...) -> Iterator[Dict[str, Any]]: ...
    def optgroups(
        self, name: str, value: List[str], attrs: Optional[_OptAttrs] = ...
    ) -> List[Tuple[Optional[str], List[Dict[str, Any]], Optional[int]]]: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...
    def create_option(
        self,
        name: str,
        value: Any,
        label: Union[int, str],
        selected: bool,
        index: int,
        subindex: Optional[int] = ...,
        attrs: Optional[_OptAttrs] = ...,
    ) -> Dict[str, Any]: ...
    def id_for_label(self, id_: str, index: str = ...) -> str: ...
    def value_from_datadict(self, data: _DataT, files: _FilesT, name: str) -> Any: ...
    def format_value(self, value: Any) -> List[str]: ...  # type: ignore

class Select(ChoiceWidget):
    input_type: Optional[str] = ...
    template_name: str = ...
    option_template_name: str = ...
    add_id_index: bool = ...
    checked_attribute: Any = ...
    option_inherits_attrs: bool = ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...
    def use_required_attribute(self, initial: Any) -> bool: ...

class NullBooleanSelect(Select):
    def __init__(self, attrs: Optional[_OptAttrs] = ...) -> None: ...
    def format_value(self, value: Any) -> str: ...  # type: ignore
    def value_from_datadict(self, data: _DataT, files: _FilesT, name: str) -> Optional[bool]: ...

class SelectMultiple(Select):
    allow_multiple_selected: bool = ...
    def value_from_datadict(self, data: _DataT, files: _FilesT, name: str) -> Any: ...
    def value_omitted_from_data(self, data: _DataT, files: _FilesT, name: str) -> bool: ...

class RadioSelect(ChoiceWidget):
    can_add_related: bool
    input_type: str = ...
    template_name: str = ...
    option_template_name: str = ...

class CheckboxSelectMultiple(ChoiceWidget):
    can_add_related: bool
    input_type: str = ...
    template_name: str = ...
    option_template_name: str = ...
    def use_required_attribute(self, initial: Any) -> bool: ...
    def value_omitted_from_data(self, data: _DataT, files: _FilesT, name: str) -> bool: ...
    def id_for_label(self, id_: str, index: Optional[str] = ...) -> str: ...

class MultiWidget(Widget):
    template_name: str = ...
    widgets: Sequence[Widget] = ...
    def __init__(
        self,
        widgets: Union[Dict[str, Union[Widget, Type[Widget]]], Sequence[Union[Widget, Type[Widget]]]],
        attrs: Optional[_OptAttrs] = ...,
    ) -> None: ...
    @property
    def is_hidden(self) -> bool: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...
    def id_for_label(self, id_: str) -> str: ...
    def value_from_datadict(self, data: _DataT, files: _FilesT, name: str) -> List[Any]: ...
    def value_omitted_from_data(self, data: _DataT, files: _FilesT, name: str) -> bool: ...
    def decompress(self, value: Any) -> Optional[Any]: ...
    media: _Getter[Media] = ...
    @property
    def needs_multipart_form(self) -> bool: ...  # type: ignore

class SplitDateTimeWidget(MultiWidget):
    supports_microseconds: bool = ...
    template_name: str = ...
    widgets: Tuple[DateInput, TimeInput]
    def __init__(
        self,
        attrs: Optional[_OptAttrs] = ...,
        date_format: Optional[str] = ...,
        time_format: Optional[str] = ...,
        date_attrs: Optional[Dict[str, str]] = ...,
        time_attrs: Optional[Dict[str, str]] = ...,
    ) -> None: ...
    def decompress(self, value: Any) -> Tuple[Optional[datetime.date], Optional[datetime.time]]: ...

class SplitHiddenDateTimeWidget(SplitDateTimeWidget):
    template_name: str = ...
    def __init__(
        self,
        attrs: Optional[_OptAttrs] = ...,
        date_format: Optional[str] = ...,
        time_format: Optional[str] = ...,
        date_attrs: Optional[Dict[str, str]] = ...,
        time_attrs: Optional[Dict[str, str]] = ...,
    ) -> None: ...

class SelectDateWidget(Widget):
    none_value: Tuple[Literal[""], str] = ...
    month_field: str = ...
    day_field: str = ...
    year_field: str = ...
    template_name: str = ...
    input_type: str = ...
    select_widget: Type[ChoiceWidget] = ...
    date_re: Any = ...
    years: Iterable[Union[int, str]] = ...
    months: Mapping[int, str] = ...
    year_none_value: Tuple[Literal[""], str] = ...
    month_none_value: Tuple[Literal[""], str] = ...
    day_none_value: Tuple[Literal[""], str] = ...
    def __init__(
        self,
        attrs: Optional[_OptAttrs] = ...,
        years: Optional[Iterable[Union[int, str]]] = ...,
        months: Optional[Mapping[int, str]] = ...,
        empty_label: Optional[Union[str, _ListOrTuple[str]]] = ...,
    ) -> None: ...
    def get_context(self, name: str, value: Any, attrs: Optional[_OptAttrs]) -> Dict[str, Any]: ...
    def format_value(self, value: Any) -> Dict[str, Union[str, int, None]]: ...  # type: ignore
    def id_for_label(self, id_: str) -> str: ...
    def value_from_datadict(self, data: _DataT, files: _FilesT, name: str) -> Union[str, None, Any]: ...
    def value_omitted_from_data(self, data: _DataT, files: _FilesT, name: str) -> bool: ...



from typing import (
    Any,
    Callable,
    Collection,
    Dict,
    Iterable,
    List,
    Mapping,
    Optional,
    Sequence,
    Sized,
    Tuple,
    TypeVar,
    Union,
    overload,
)

from django.template.base import FilterExpression, Origin, Parser, Token
from django.template.context import Context
from django.utils.safestring import SafeString

from .base import Node, Template

class InvalidTemplateLibrary(Exception): ...

_C = TypeVar("_C", bound=Callable[..., Any])

class Library:
    filters: Dict[str, Callable] = ...
    tags: Dict[str, Callable] = ...
    def __init__(self) -> None: ...
    @overload
    def tag(self, name: _C) -> _C: ...
    @overload
    def tag(self, name: str, compile_function: _C) -> _C: ...
    @overload
    def tag(self, name: Optional[str] = ..., compile_function: None = ...) -> Callable[[_C], _C]: ...
    def tag_function(self, func: _C) -> _C: ...
    @overload
    def filter(self, name: _C, filter_func: None = ..., **flags: Any) -> _C: ...
    @overload
    def filter(self, name: Optional[str], filter_func: _C, **flags: Any) -> _C: ...
    @overload
    def filter(self, name: Optional[str] = ..., filter_func: None = ..., **flags: Any) -> Callable[[_C], _C]: ...
    @overload
    def simple_tag(self, func: _C) -> _C: ...
    @overload
    def simple_tag(self, takes_context: Optional[bool] = ..., name: Optional[str] = ...) -> Callable[[_C], _C]: ...
    def inclusion_tag(
        self,
        filename: Union[Template, str],
        func: Optional[Callable] = ...,
        takes_context: Optional[bool] = ...,
        name: Optional[str] = ...,
    ) -> Callable[[_C], _C]: ...

class TagHelperNode(Node):
    func: Any = ...
    takes_context: Any = ...
    args: Any = ...
    kwargs: Any = ...
    def __init__(
        self,
        func: Callable,
        takes_context: Optional[bool],
        args: List[FilterExpression],
        kwargs: Dict[str, FilterExpression],
    ) -> None: ...
    def get_resolved_arguments(self, context: Context) -> Tuple[List[int], Dict[str, Union[SafeString, int]]]: ...

class SimpleNode(TagHelperNode):
    args: List[FilterExpression]
    func: Callable
    kwargs: Dict[str, FilterExpression]
    origin: Origin
    takes_context: Optional[bool]
    token: Token
    target_var: Optional[str] = ...
    def __init__(
        self,
        func: Callable,
        takes_context: Optional[bool],
        args: List[FilterExpression],
        kwargs: Dict[str, FilterExpression],
        target_var: Optional[str],
    ) -> None: ...

class InclusionNode(TagHelperNode):
    args: List[FilterExpression]
    func: Callable
    kwargs: Dict[str, FilterExpression]
    origin: Origin
    takes_context: Optional[bool]
    token: Token
    filename: Union[Template, str] = ...
    def __init__(
        self,
        func: Callable,
        takes_context: Optional[bool],
        args: List[FilterExpression],
        kwargs: Dict[str, FilterExpression],
        filename: Optional[Union[Template, str]],
    ) -> None: ...

def parse_bits(
    parser: Parser,
    bits: Iterable[str],
    params: Sequence[str],
    varargs: Optional[str],
    varkw: Optional[str],
    defaults: Optional[Sized],
    kwonly: Collection[str],
    kwonly_defaults: Optional[Mapping[str, int]],
    takes_context: Optional[bool],
    name: str,
) -> Tuple[List[FilterExpression], Dict[str, FilterExpression]]: ...
def import_library(name: str) -> Library: ...



from typing import Iterable, Iterator, Optional

from django.core.files.storage import Storage

def matches_patterns(path: str, patterns: Iterable[str]) -> bool: ...
def get_files(
    storage: Storage, ignore_patterns: Optional[Iterable[str]] = ..., location: str = ...
) -> Iterator[str]: ...
def check_settings(base_url: Optional[str] = ...) -> None: ...



import sys
from typing import Any, Dict, Iterable, Iterator, List, Optional, Sequence, Tuple, Type, overload

from django.core.checks.messages import CheckMessage
from django.core.files.storage import FileSystemStorage, Storage
from django.utils._os import _PathCompatible

if sys.version_info < (3, 8):
    from typing_extensions import Literal
else:
    from typing import Literal

searched_locations: Any

class BaseFinder:
    def check(self, **kwargs: Any) -> List[CheckMessage]: ...
    @overload
    def find(self, path: _PathCompatible, all: Literal[False] = False) -> Optional[_PathCompatible]: ...  # type: ignore
    @overload
    def find(self, path: _PathCompatible, all: Literal[True] = ...) -> List[_PathCompatible]: ...
    def list(self, ignore_patterns: Optional[Iterable[str]]) -> Iterable[Any]: ...

class FileSystemFinder(BaseFinder):
    locations: List[Tuple[str, _PathCompatible]] = ...
    storages: Dict[_PathCompatible, Any] = ...
    def __init__(self, app_names: Sequence[str] = ..., *args: Any, **kwargs: Any) -> None: ...
    def find_location(
        self, root: _PathCompatible, path: str, prefix: Optional[str] = ...
    ) -> Optional[_PathCompatible]: ...
    @overload
    def find(self, path: _PathCompatible, all: Literal[False] = False) -> Optional[_PathCompatible]: ...  # type: ignore
    @overload
    def find(self, path: _PathCompatible, all: Literal[True] = ...) -> List[_PathCompatible]: ...
    def list(self, ignore_patterns: Optional[Iterable[str]]) -> Iterable[Any]: ...

class AppDirectoriesFinder(BaseFinder):
    storage_class: Type[FileSystemStorage] = ...
    source_dir: str = ...
    apps: List[str] = ...
    storages: Dict[_PathCompatible, FileSystemStorage] = ...
    def __init__(self, app_names: Optional[Iterable[str]] = ..., *args: Any, **kwargs: Any) -> None: ...
    def find_in_app(self, app: str, path: _PathCompatible) -> Optional[_PathCompatible]: ...
    @overload
    def find(self, path: _PathCompatible, all: Literal[False] = False) -> Optional[_PathCompatible]: ...  # type: ignore
    @overload
    def find(self, path: _PathCompatible, all: Literal[True] = ...) -> List[_PathCompatible]: ...
    def list(self, ignore_patterns: Optional[Iterable[str]]) -> Iterable[Any]: ...

class BaseStorageFinder(BaseFinder):
    storage: Storage = ...
    def __init__(self, storage: Optional[Storage] = ..., *args: Any, **kwargs: Any) -> None: ...
    @overload
    def find(self, path: _PathCompatible, all: Literal[False] = False) -> Optional[_PathCompatible]: ...  # type: ignore
    @overload
    def find(self, path: _PathCompatible, all: Literal[True] = ...) -> List[_PathCompatible]: ...
    def list(self, ignore_patterns: Optional[Iterable[str]]) -> Iterable[Any]: ...

class DefaultStorageFinder(BaseStorageFinder):
    storage: Storage = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

@overload
def find(path: _PathCompatible, all: Literal[False] = False) -> Optional[_PathCompatible]: ...  # type: ignore
@overload
def find(path: _PathCompatible, all: Literal[True] = ...) -> List[_PathCompatible]: ...
def get_finders() -> Iterator[BaseFinder]: ...
@overload
def get_finder(
    import_path: Literal["django.contrib.staticfiles.finders.FileSystemFinder"],
) -> FileSystemFinder: ...
@overload
def get_finder(
    import_path: Literal["django.contrib.staticfiles.finders.AppDirectoriesFinder"],
) -> AppDirectoriesFinder: ...



from typing import Any, Optional, Sequence

from django.apps.config import AppConfig
from django.core.checks.messages import Error

E001: Error

def check_async_unsafe(app_configs: Optional[Sequence[AppConfig]], **kwargs: Any) -> Sequence[Error]: ...



from typing import Any, Optional, Sequence

from django.apps.config import AppConfig

from . import Error

E001: Error = ...
E002: Error = ...
E003: Error = ...
E004: Error = ...

def check_setting_language_code(app_configs: Optional[Sequence[AppConfig]], **kwargs: Any) -> Sequence[Error]: ...
def check_setting_languages(app_configs: Optional[Sequence[AppConfig]], **kwargs: Any) -> Sequence[Error]: ...
def check_setting_languages_bidi(app_configs: Optional[Sequence[AppConfig]], **kwargs: Any) -> Sequence[Error]: ...
def check_language_settings_consistent(
    app_configs: Optional[Sequence[AppConfig]], **kwargs: Any
) -> Sequence[Error]: ...



import sys
import types
from contextlib import ContextDecorator
from datetime import date
from datetime import datetime as datetime
from datetime import time
from datetime import timedelta as timedelta
from datetime import timezone
from datetime import tzinfo as tzinfo
from typing import Any, Optional, Type, Union, overload

if sys.version_info < (3, 8):
    from typing_extensions import Literal
else:
    from typing import Literal

if sys.version_info < (3, 10):
    from typing_extensions import TypeGuard
else:
    from typing import TypeGuard

import pytz
from pytz import BaseTzInfo

_PytzTzInfoT = Union[pytz.tzinfo.BaseTzInfo, pytz._FixedOffset]
_TzInfoT = Union[_PytzTzInfoT, tzinfo]

utc: Any = ...

def get_fixed_timezone(offset: Union[timedelta, int]) -> timezone: ...
def get_default_timezone() -> BaseTzInfo: ...
def get_default_timezone_name() -> str: ...

# Strictly speaking, it is possible to activate() a non-pytz timezone,
# in which case BaseTzInfo is incorrect. However, this is unlikely,
# so we use it anyway, to keep things ergonomic for most users.
def get_current_timezone() -> BaseTzInfo: ...
def get_current_timezone_name() -> str: ...
def activate(timezone: Union[_TzInfoT, str]) -> None: ...
def deactivate() -> None: ...

class override(ContextDecorator):
    timezone: Optional[Union[str, _TzInfoT]] = ...
    old_timezone: Optional[_TzInfoT] = ...
    def __init__(self, timezone: Optional[Union[str, _TzInfoT]]) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[types.TracebackType],
    ) -> None: ...

def localtime(value: Optional[datetime] = ..., timezone: Optional[_TzInfoT] = ...) -> datetime: ...
def localdate(value: Optional[datetime] = ..., timezone: Optional[_TzInfoT] = ...) -> date: ...
def now() -> datetime: ...
@overload
def is_aware(value: time) -> Literal[False]: ...
@overload
def is_aware(value: datetime) -> bool: ...
@overload
def is_naive(value: time) -> Literal[True]: ...
@overload
def is_naive(value: datetime) -> bool: ...
def make_aware(value: datetime, timezone: Optional[_TzInfoT] = ..., is_dst: Optional[bool] = ...) -> datetime: ...
def make_naive(value: datetime, timezone: Optional[_TzInfoT] = ...) -> datetime: ...
def _is_pytz_zone(tz: _TzInfoT) -> TypeGuard[_PytzTzInfoT]: ...



import sys
from typing import Any, Mapping, Optional, Sequence, Union

from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.migrations.state import StateApps
from django.utils.datastructures import _ListOrTuple

from .base import Operation

if sys.version_info < (3, 8):
    from typing_extensions import Literal, Protocol
else:
    from typing import Literal, Protocol

class SeparateDatabaseAndState(Operation):
    database_operations: Sequence[Operation] = ...
    state_operations: Sequence[Operation] = ...
    def __init__(
        self, database_operations: Sequence[Operation] = ..., state_operations: Sequence[Operation] = ...
    ) -> None: ...

class RunSQL(Operation):
    noop: Literal[""] = ...
    sql: Union[str, _ListOrTuple[str]] = ...
    reverse_sql: Optional[Union[str, _ListOrTuple[str]]] = ...
    state_operations: Sequence[Operation] = ...
    hints: Mapping[str, Any] = ...
    def __init__(
        self,
        sql: Union[str, _ListOrTuple[str]],
        reverse_sql: Optional[Union[str, _ListOrTuple[str]]] = ...,
        state_operations: Sequence[Operation] = ...,
        hints: Optional[Mapping[str, Any]] = ...,
        elidable: bool = ...,
    ) -> None: ...

class _CodeCallable(Protocol):
    def __call__(self, __state_apps: StateApps, __shema_editor: BaseDatabaseSchemaEditor) -> None: ...

class RunPython(Operation):
    code: _CodeCallable = ...
    reverse_code: Optional[_CodeCallable] = ...
    hints: Mapping[str, Any] = ...
    def __init__(
        self,
        code: _CodeCallable,
        reverse_code: Optional[_CodeCallable] = ...,
        atomic: Optional[bool] = ...,
        hints: Optional[Mapping[str, Any]] = ...,
        elidable: bool = ...,
    ) -> None: ...
    @staticmethod
    def noop(apps: StateApps, schema_editor: BaseDatabaseSchemaEditor) -> None: ...



from typing import Any, Dict, Generic, Iterator, Mapping, Optional, Sequence, Type, TypeVar

class ConnectionProxy:
    def __init__(self, connections: Mapping[str, Any], alias: str) -> None: ...
    def __getattr__(self, item: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __delattr__(self, name: str) -> None: ...
    def __contains__(self, key: str) -> bool: ...
    def __eq__(self, other: Any) -> bool: ...

class ConnectionDoesNotExist(Exception): ...

_T = TypeVar("_T")

class BaseConnectionHandler(Generic[_T]):
    settings_name: Optional[str] = ...
    exception_class: Type[Exception] = ...
    thread_critical: bool = ...
    def __init__(self, settings: Optional[Any] = ...) -> None: ...
    @property
    def settings(self) -> Dict[str, Any]: ...
    def configure_settings(self, settings: Optional[Dict[str, Any]]) -> Dict[str, Any]: ...
    def create_connection(self, alias: str) -> _T: ...
    def __getitem__(self, alias: str) -> _T: ...
    def __setitem__(self, key: str, value: _T) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def all(self) -> Sequence[_T]: ...



from contextlib import contextmanager
from typing import Any, Generator, Optional, Type

from django.db.backends.base.base import BaseDatabaseWrapper as BaseDatabaseWrapper

from .client import DatabaseClient
from .creation import DatabaseCreation
from .features import DatabaseFeatures
from .introspection import DatabaseIntrospection
from .operations import DatabaseOperations
from .validation import DatabaseValidation

@contextmanager
def wrap_oracle_errors() -> Generator[None, None, None]: ...

class _UninitializedOperatorsDescriptor:
    def __get__(self, instance: Any, cls: Optional[Any] = ...): ...

class DatabaseWrapper(BaseDatabaseWrapper):
    client: DatabaseClient
    creation: DatabaseCreation
    features: DatabaseFeatures
    introspection: DatabaseIntrospection
    validation: DatabaseValidation
    ops: DatabaseOperations

    client_class: Type[DatabaseClient]
    creation_class: Type[DatabaseCreation]
    features_class: Type[DatabaseFeatures]
    introspection_class: Type[DatabaseIntrospection]
    ops_class: Type[DatabaseOperations]
    validation_class: Type[DatabaseValidation]

    vendor: str = ...
    display_name: str = ...
    data_types: Any = ...
    data_type_check_constraints: Any = ...
    operators: Any = ...
    pattern_esc: str = ...
    Database: Any = ...
    SchemaEditorClass: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def get_connection_params(self): ...
    def get_new_connection(self, conn_params: Any): ...
    pattern_ops: Any = ...
    def init_connection_state(self) -> None: ...
    def create_cursor(self, name: Optional[Any] = ...): ...
    def check_constraints(self, table_names: Optional[Any] = ...) -> None: ...
    def is_usable(self): ...
    @property
    def cx_oracle_version(self): ...
    @property
    def oracle_version(self): ...

class OracleParam:
    force_bytes: Any = ...
    input_size: Any = ...
    def __init__(self, param: Any, cursor: Any, strings_only: bool = ...) -> None: ...

class VariableWrapper:
    var: Any = ...
    def __init__(self, var: Any) -> None: ...
    def bind_parameter(self, cursor: Any): ...
    def __getattr__(self, key: Any): ...
    def __setattr__(self, key: Any, value: Any) -> None: ...

class FormatStylePlaceholderCursor:
    charset: str = ...
    cursor: Any = ...
    def __init__(self, connection: Any) -> None: ...
    def execute(self, query: Any, params: Optional[Any] = ...): ...
    def executemany(self, query: Any, params: Optional[Any] = ...): ...
    def close(self) -> None: ...
    def var(self, *args: Any): ...
    def arrayvar(self, *args: Any): ...
    def __getattr__(self, attr: Any): ...
    def __iter__(self) -> Any: ...



from typing import IO, Any, Iterator, Union

from django.core.serializers.base import DeserializedObject
from django.core.serializers.python import Serializer as PythonSerializer
from django.db.models.fields import Field
from yaml import CSafeDumper as SafeDumper

class DjangoSafeDumper(SafeDumper):
    def represent_decimal(self, data: Any): ...
    def represent_ordered_dict(self, data: Any): ...

class Serializer(PythonSerializer):
    internal_use_only: bool = ...
    def handle_field(self, obj: Any, field: Field) -> None: ...
    def end_serialization(self) -> None: ...
    def getvalue(self) -> Any: ...

def Deserializer(
    stream_or_string: Union[bytes, str, IO[bytes], IO[str]], **options: Any
) -> Iterator[DeserializedObject]: ...



from typing import IO, Any, Dict, Iterator, Union

from django.core.serializers.base import DeserializedObject
from django.core.serializers.python import Serializer as PythonSerializer

class Serializer(PythonSerializer):
    json_kwargs: Dict[str, Any]

def Deserializer(
    stream_or_string: Union[IO[bytes], IO[str], bytes, str], **options: Any
) -> Iterator[DeserializedObject]: ...



ABSTRACT_USER_MODEL_FULLNAME = "django.contrib.auth.models.AbstractUser"
PERMISSION_MIXIN_CLASS_FULLNAME = "django.contrib.auth.models.PermissionsMixin"
MODEL_CLASS_FULLNAME = "django.db.models.base.Model"
FIELD_FULLNAME = "django.db.models.fields.Field"
CHAR_FIELD_FULLNAME = "django.db.models.fields.CharField"
ARRAY_FIELD_FULLNAME = "django.contrib.postgres.fields.array.ArrayField"
AUTO_FIELD_FULLNAME = "django.db.models.fields.AutoField"
GENERIC_FOREIGN_KEY_FULLNAME = "django.contrib.contenttypes.fields.GenericForeignKey"
FOREIGN_KEY_FULLNAME = "django.db.models.fields.related.ForeignKey"
ONETOONE_FIELD_FULLNAME = "django.db.models.fields.related.OneToOneField"
MANYTOMANY_FIELD_FULLNAME = "django.db.models.fields.related.ManyToManyField"
DUMMY_SETTINGS_BASE_CLASS = "django.conf._DjangoConfLazyObject"

QUERYSET_CLASS_FULLNAME = "django.db.models.query._QuerySet"
BASE_MANAGER_CLASS_FULLNAME = "django.db.models.manager.BaseManager"
MANAGER_CLASS_FULLNAME = "django.db.models.manager.Manager"
RELATED_MANAGER_CLASS = "django.db.models.manager.RelatedManager"

WITH_ANNOTATIONS_FULLNAME = "django_stubs_ext.WithAnnotations"
ANNOTATIONS_FULLNAME = "django_stubs_ext.annotations.Annotations"

BASEFORM_CLASS_FULLNAME = "django.forms.forms.BaseForm"
FORM_CLASS_FULLNAME = "django.forms.forms.Form"
MODELFORM_CLASS_FULLNAME = "django.forms.models.ModelForm"

FORM_MIXIN_CLASS_FULLNAME = "django.views.generic.edit.FormMixin"

MANAGER_CLASSES = {
    MANAGER_CLASS_FULLNAME,
    BASE_MANAGER_CLASS_FULLNAME,
}

RELATED_FIELDS_CLASSES = {FOREIGN_KEY_FULLNAME, ONETOONE_FIELD_FULLNAME, MANYTOMANY_FIELD_FULLNAME}

MIGRATION_CLASS_FULLNAME = "django.db.migrations.migration.Migration"
OPTIONS_CLASS_FULLNAME = "django.db.models.options.Options"
HTTPREQUEST_CLASS_FULLNAME = "django.http.request.HttpRequest"
QUERYDICT_CLASS_FULLNAME = "django.http.request.QueryDict"

COMBINABLE_EXPRESSION_FULLNAME = "django.db.models.expressions.Combinable"
F_EXPRESSION_FULLNAME = "django.db.models.expressions.F"

ANY_ATTR_ALLOWED_CLASS_FULLNAME = "django_stubs_ext.AnyAttrAllowed"



from io import IOBase
from typing import Any, Dict, Tuple, Type

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.utils import CursorDebugWrapper as BaseCursorDebugWrapper

from .client import DatabaseClient
from .creation import DatabaseCreation
from .features import DatabaseFeatures
from .introspection import DatabaseIntrospection
from .operations import DatabaseOperations

def psycopg2_version() -> Tuple[int, ...]: ...

PSYCOPG2_VERSION: Tuple[int, ...] = ...

class DatabaseWrapper(BaseDatabaseWrapper):
    client: DatabaseClient
    creation: DatabaseCreation
    features: DatabaseFeatures
    introspection: DatabaseIntrospection
    ops: DatabaseOperations

    client_class: Type[DatabaseClient]
    creation_class: Type[DatabaseCreation]
    features_class: Type[DatabaseFeatures]
    introspection_class: Type[DatabaseIntrospection]
    ops_class: Type[DatabaseOperations]

    operators: Dict[str, str] = ...
    pattern_esc: str = ...
    pattern_ops: Dict[str, str] = ...

    # PostgreSQL backend-specific attributes.
    _named_cursor_idx: int = ...
    @property
    def pg_version(self) -> int: ...

class CursorDebugWrapper(BaseCursorDebugWrapper):
    def copy_expert(self, sql: str, file: IOBase, *args: Any): ...
    def copy_to(self, file: IOBase, table: str, *args: Any, **kwargs: Any): ...



from typing import Any, Callable, Dict, List, Optional, Set, Type

from django.contrib.gis.db.backends.base.operations import BaseSpatialOperations as BaseSpatialOperations
from django.contrib.gis.db.backends.utils import SpatialOperator
from django.contrib.gis.geos.geometry import GEOSGeometryBase
from django.db.backends.mysql.operations import DatabaseOperations as DatabaseOperations

class MySQLOperations(BaseSpatialOperations, DatabaseOperations):
    name: str = ...
    geom_func_prefix: str = ...
    Adapter: Any = ...
    @property
    def mariadb(self) -> bool: ...
    @property
    def mysql(self) -> bool: ...  # type: ignore
    @property
    def select(self) -> str: ...  # type: ignore
    @property
    def from_text(self) -> str: ...  # type: ignore
    @property
    def gis_operators(self) -> Dict[str, SpatialOperator]: ...
    disallowed_aggregates: Any = ...
    @property
    def unsupported_functions(self) -> Set[str]: ...  # type: ignore
    def geo_db_type(self, f: Any) -> Any: ...
    def get_distance(self, f: Any, value: Any, lookup_type: Any) -> List[Any]: ...
    def get_geometry_converter(self, expression: Any) -> Callable[[Any, Any, Any], Optional[GEOSGeometryBase]]: ...



import sys
from collections import namedtuple
from typing import Iterator, Optional, Tuple, Union

from django.db.migrations.state import ModelState, ProjectState
from django.db.models import Field

if sys.version_info < (3, 8):
    from typing_extensions import Literal
else:
    from typing import Literal

def resolve_relation(model, app_label: Optional[str] = ..., model_name: Optional[str] = ...) -> Tuple[str, str]: ...

FieldReference = namedtuple("FieldReference", ["to", "through"])

def field_references(
    model_tuple: Tuple[str, str],
    field: Field,
    reference_model_tuple: Tuple[str, str],
    reference_field_name: Optional[str] = ...,
    reference_field: Optional[Field] = ...,
) -> Union[Literal[False], FieldReference]: ...
def get_references(
    state: ProjectState,
    model_tuple: Tuple[str, str],
    field_tuple: Union[Tuple[()], Tuple[str, Field]] = ...,
) -> Iterator[Tuple[ModelState, str, Field, FieldReference]]: ...
def field_is_referenced(state: ProjectState, model_tuple: Tuple[str, str], field_tuple: Tuple[str, Field]) -> bool: ...



from typing import Any, Iterable, Optional, Sequence

from django.core.checks import CheckMessage

def check_database_backends(databases: Optional[Iterable[str]] = ..., **kwargs: Any) -> Sequence[CheckMessage]: ...



import sys
from typing import Dict, Final, Pattern, Tuple

if sys.version_info < (3, 8):
    from typing_extensions import Literal
else:
    from typing import Literal

GET_ITERATOR_CHUNK_SIZE: Final[int] = ...

MULTI: Literal["multi"] = ...
SINGLE: Literal["single"] = ...
CURSOR: Literal["cursor"] = ...
NO_RESULTS: Literal["no results"] = ...

ORDER_PATTERN: Pattern = ...
ORDER_DIR: Dict[str, Tuple[str, str]] = ...

INNER: Literal["INNER JOIN"] = ...
LOUTER: Literal["LEFT OUTER JOIN"] = ...



import os
from typing import Any, Iterable, List, Optional, Sequence, Set, Tuple, Type, Union

from django.apps.config import AppConfig
from django.db.models.base import Model

def popen_wrapper(args: List[str], stdout_encoding: str = ...) -> Tuple[str, str, int]: ...
def handle_extensions(extensions: Iterable[str]) -> Set[str]: ...
def find_command(
    cmd: str,
    path: Optional[Union[str, Iterable[str]]] = ...,
    pathext: Optional[Union[str, Iterable[str]]] = ...,
) -> Optional[str]: ...
def get_random_secret_key() -> str: ...
def parse_apps_and_model_labels(labels: Iterable[str]) -> Tuple[Set[Type[Model]], Set[AppConfig]]: ...
def get_command_line_option(argv: Sequence[Any], option: Any) -> Optional[Any]: ...
def normalize_path_patterns(patterns: Iterable[str]) -> List[str]: ...
def is_ignored_path(path: Union[str, os.PathLike], ignore_patterns: Iterable[str]) -> bool: ...
